package hamster.kalah;

import hamster.*;

/**
 * Repraesentation eines als Schiedsrichter fungierenden Hamsters beim
 * Kalah-Spiel
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (29.01.2006)
 * 
 */
public class SchiedsrichterHamster extends AllroundHamster {

	private Spielbrett brett; // internes Spielbrett

	private Spielregeln regeln; // Spielregeln

	private Spieler spielerA, spielerB; // die Spieler

	private Spieler aktuellerSpieler; // aktueller Spieler

	private int letzteMulde; // im letzten Spielzug genannte Mulde

	/**
	 * initialisiert einen SchiedsrichterHamster
	 */
	public SchiedsrichterHamster() {
		super(0, 0, Hamster.OST, Spielbrett.ANZAHL_KOERNER);
	}

	/**
	 * SchiedsrichterHamster bereitet das Spiel vor; insbesondere wird das
	 * Spielfeld mit Koernern aufgebaut
	 * 
	 * @param spielerA
	 *            Spieler A
	 * @param spielerB
	 *            Spieler B
	 */
	public void initialisiereSpiel(Spieler spielerA, Spieler spielerB) {
		this.spielerA = spielerA;
		this.spielerB = spielerB;
		this.brett = new Spielbrett();
		this.regeln = new Spielregeln(this.brett);
		this.spielerA.teileSpielregelnMit(this.regeln);
		this.spielerB.teileSpielregelnMit(this.regeln);
		this.aktuellerSpieler = null;
		this.erstelleSpielfeld();
	}

	/**
	 * liefert Spieler A
	 * 
	 * @return Spieler A
	 */
	public Spieler getSpielerA() {
		return this.spielerA;
	}

	/**
	 * liefert Spieler B
	 * 
	 * @return Spieler B
	 */
	public Spieler getSpielerB() {
		return this.spielerB;
	}

	/**
	 * ueberprueft den Spielzug eines Spielers
	 * 
	 * @param spieler
	 *            der Spieler, dessen Spielzug kontrolliert werden soll
	 * @param zug
	 *            der zu kontrollierende Spielzug
	 * @return true, falls der Spielzug ok war; false sonst
	 */
	public boolean istSpielzugOk(Spieler spieler, Spielzug zug) {

		// merkt sich den aktuellen Spieler und Spielzug
		this.aktuellerSpieler = spieler;

		// ueberprueft den Spielzug auf dem internen Brett
		if (!this.regeln.istSpielzugOk(spieler.istSpielerA(), zug)) {
			return false;
		}

		// fuehrt den Spielzug auf dem internen Brett aus;
		// merkt sich die Mulde, in die das letzte Korn gelegt wurde
		this.letzteMulde = this.brett.fuehreSpielzugAus(spieler.istSpielerA(),
				zug);

		// nun ueberprueft der Schiedsrichter-Hamster noch, ob auch
		// das Hamster-Territorium gemaess der Regeln in Ordnung ist;
		return this.kontrolliereTerritorium();
	}

	/**
	 * ueberprueft, ob ein Spiel beendet ist
	 * 
	 * @return true, falls das Spiel beendet ist; false sonst
	 */
	public boolean istSpielBeendet() {
		boolean ende = this.regeln.istSpielBeendet(this.werIstAmZug()
				.istSpielerA());
		if (ende) {
			// auf dem internen Brett wird der Endzustand hergestellt
			this.brett.spielBeenden();
		}
		return ende;
	}

	/**
	 * ueberprueft Endzustand von Spieler A
	 * 
	 * @return true, falls der Endzustand von Spieler A ok ist; false sonst
	 */
	public boolean istEndzustandVonSpielerAOk() {
		this.vor(Spielbrett.ANZAHL_MULDEN_PRO_SPIELER);
		this.kehrt();
		for (int i = 0; i < Spielbrett.ANZAHL_MULDEN_PRO_SPIELER; i++) {
			if (Territorium.getAnzahlKoerner(this.getReihe(), this.getSpalte()) != 0) {
				return false;
			}
			this.vor();
		}
		this.linksUm();
		this.vor();
		if (Territorium.getAnzahlKoerner(this.getReihe(), this.getSpalte()) != this.brett
				.liefereAnzahlKoerner(Spielbrett.KALAH_A)) {
			return false;
		}
		this.gehZumAusgangspunkt();
		return true;
	}

	/**
	 * ueberprueft Endzustand von Spieler B
	 * 
	 * @return true, falls der Endzustand von Spieler B ok ist; false sonst
	 */
	public boolean istEndzustandVonSpielerBOk() {
		this.rechtsUm();
		this.vor(2);
		this.linksUm();
		this.vor(1);
		for (int i = 0; i < Spielbrett.ANZAHL_MULDEN_PRO_SPIELER; i++) {
			if (Territorium.getAnzahlKoerner(this.getReihe(), this.getSpalte()) != 0) {
				return false;
			}
			this.vor();
		}
		this.linksUm();
		this.vor();
		if (Territorium.getAnzahlKoerner(this.getReihe(), this.getSpalte()) != this.brett
				.liefereAnzahlKoerner(Spielbrett.KALAH_B)) {
			return false;
		}
		this.gehZumAusgangspunkt();
		return true;
	}

	/**
	 * liefert den Sieger eines beendeten Spieles; liefert null, wenn das Spiel
	 * mit einem Unentschieden endete
	 * 
	 * @return Sieger eines beendeten Spieles bzw. null, wenn das Spiel mit
	 *         einem Unentschieden endete
	 */
	public Spieler liefereSieger() {
		if (this.regeln.istUnentschieden()) {
			return null;
		} else if (this.regeln.istSpielerASieger()) {
			return this.spielerA;
		} else {
			return this.spielerB;
		}
	}

	/**
	 * liefert den naechsten Spieler, der am Zug ist
	 * 
	 * @return der naechste Spieler, der am Zug ist
	 */
	public Spieler werIstAmZug() {
		// es beginnt Spieler A
		if (this.aktuellerSpieler == null) {
			return this.spielerA;
		}

		if (this.aktuellerSpieler.istSpielerA()
				&& this.letzteMulde == Spielbrett.KALAH_A
				|| !this.aktuellerSpieler.istSpielerA()
				&& this.letzteMulde == Spielbrett.KALAH_B) {

			// die letzte Kugel landete im eigenen Kalah, d.h.
			// der Spieler muss nochmal ziehen
			return this.aktuellerSpieler;
		} else {

			// der Gegner ist an der Reihe
			return this.liefereGegner(this.aktuellerSpieler);
		}
	}

	/**
	 * liefert den gegnerischen Spieler des uebergebenen Spielers
	 * 
	 * @param spieler
	 *            der Spieler, dessen Gegner ermittelt werden soll
	 * @return der gegnerische Spieler des uebergebenen Spielers
	 */
	public Spieler liefereGegner(Spieler spieler) {
		if (spieler == this.spielerA) {
			return spielerB;
		} else {
			return spielerA;
		}
	}

	/**
	 * liefert die Punkte von Spieler A
	 * 
	 * @return die Punkte von Spieler A
	 */
	public int getPunkteA() {
		return this.brett.liefereAnzahlKoernerInKalahA();
	}

	/**
	 * liefert die Punkte von Spieler A
	 * 
	 * @return die Punkte von Spieler A
	 */
	public int getPunkteB() {
		return this.brett.liefereAnzahlKoernerInKalahB();
	}

	// interne Methoden

	/**
	 * laeuft zur Kachel (0/0) und schaut nach Osten
	 */
	protected void gehZumAusgangspunkt() {
		this.laufeZuKachel(0, 0);
		this.setzeBlickrichtung(Hamster.OST);
	}

	/**
	 * der Schiedsrichter-Hamster ueberprueft, ob auch das Hamster-Territorium
	 * gemaess der Regeln in Ordnung ist; d.h. ob der Zustand des Territoriums
	 * und des internen Brettes identisch sind
	 * 
	 * @return true, falls das Territotrium ok ist; false sonst
	 */
	protected boolean kontrolliereTerritorium() {
		this.vor();
		for (int i = Spielbrett.ANZAHL_MULDEN_PRO_SPIELER - 1; i >= 0; i--) {
			if (this.brett.liefereAnzahlKoerner(i) != Territorium
					.getAnzahlKoerner(this.getReihe(), this.getSpalte())) {
				return false;
			}
			this.vor();
		}
		this.rechtsUm();
		this.vor();
		if (this.brett.liefereAnzahlKoerner(Spielbrett.KALAH_B) != Territorium
				.getAnzahlKoerner(this.getReihe(), this.getSpalte())) {
			return false;
		}
		this.vor();
		this.rechtsUm();
		this.vor();
		for (int i = Spielbrett.KALAH_B - 1; i >= Spielbrett.KALAH_A + 1; i--) {
			if (this.brett.liefereAnzahlKoerner(i) != Territorium
					.getAnzahlKoerner(this.getReihe(), this.getSpalte())) {
				return false;
			}
			this.vor();
		}
		this.rechtsUm();
		this.vor();
		if (this.brett.liefereAnzahlKoerner(Spielbrett.KALAH_A) != Territorium
				.getAnzahlKoerner(this.getReihe(), this.getSpalte())) {
			return false;
		}

		// und zurueck zum Ausgangspunkt
		this.vor();
		this.rechtsUm();

		return true;
	}

	/**
	 * baut das Spielfeld auf
	 */
	protected void erstelleSpielfeld() {
		// sorgt dafuer, dass in den Mulden die exakte Anzahl an Koernern
		// liegt
		this.vor();
		for (int i = Spielbrett.ANZAHL_MULDEN_PRO_SPIELER - 1; i >= 0; i--) {
			int anzahl = Territorium.getAnzahlKoerner(this.getReihe(), this
					.getSpalte());
			// zu viele Koerner?
			while (anzahl > Spielbrett.ANZAHL_KOERNER_PRO_MULDE) {
				this.nimm();
				anzahl = anzahl - 1;
			}
			// zu wenige Koerner?
			while (anzahl < Spielbrett.ANZAHL_KOERNER_PRO_MULDE) {
				this.gib();
				anzahl = anzahl + 1;
			}
			this.vor();
		}
		this.rechtsUm();
		this.vor();

		// Kalah B leer machen
		this.nimmAlle();

		this.vor();
		this.rechtsUm();
		this.vor();
		for (int i = Spielbrett.KALAH_B - 1; i >= Spielbrett.KALAH_A + 1; i--) {
			int anzahl = Territorium.getAnzahlKoerner(this.getReihe(), this
					.getSpalte());
			// zu viele Koerner?
			while (anzahl > Spielbrett.ANZAHL_KOERNER_PRO_MULDE) {
				this.nimm();
				anzahl = anzahl - 1;
			}
			// zu wenige Koerner?
			while (anzahl < Spielbrett.ANZAHL_KOERNER_PRO_MULDE) {
				this.gib();
				anzahl = anzahl + 1;
			}
			this.vor();
		}
		this.rechtsUm();
		this.vor();

		// Kalah A leer machen
		this.nimmAlle();

		// und zurueck zum Ausgangspunkt
		this.vor();
		this.rechtsUm();
	}
}
