package hamster.kalah;

/**
 * Repraesentation der Spielregeln beim Kalah-Spiel
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (29.01.2006)
 * 
 */
public class Spielregeln {

	private Spielbrett brett; // das zu kontrollierende Brett

	/**
	 * initialisiert die Regeln; uebergeben wird das zu kontrollierende
	 * Spielbrett
	 * 
	 * @param brett
	 *            das zu kontrollierende Spielbrett
	 */
	public Spielregeln(Spielbrett brett) {
		this.brett = brett;
	}

	/**
	 * kontrolliert, ob der angegebene Spielzug des angegebenen Spielers korrekt
	 * ist
	 * 
	 * @param istSpielerA
	 *            Spieler des Spielzugs
	 * @param zug
	 *            der zu kontrollierende Spielzug
	 * @return true, falls der Spielzug korrekt ist, false sonst
	 */
	public boolean istSpielzugOk(boolean istSpielerA, Spielzug zug) {
		if (zug == null) {
			return false;
		}

		// es muss eine Mulde sein, die dem Spieler gehoert
		int nummer = zug.liefereMulde();
		if (istSpielerA) {
			if (!(0 <= nummer && nummer < Spielbrett.KALAH_A)) {
				return false;
			}
		} else {
			if (!(Spielbrett.KALAH_A < nummer && nummer < Spielbrett.KALAH_B)) {
				return false;
			}
		}

		// die Mulde darf nicht leer sein
		if (this.brett.liefereAnzahlKoerner(nummer) == 0) {
			return false;
		}

		// Spielzug ist ok
		return true;
	}

	/**
	 * liefert die Information, ob A als naechster ziehen muss; uebergeben wird
	 * die Information, ob A als letzter gezogen hat, sowie die Nummer der
	 * Mulde, in der der letzte Zug geendet hat
	 * 
	 * @param aLetzterSpieler
	 *            true bedeutet, dass Spieler A als letztes am Zug war; false
	 *            bedeutet, das Spieler B als letztes am Zug war
	 * @param mulde
	 *            Nummer der Mulde, in der der letzte Zug geendet hat
	 * @return true, falls Spieler A als naechstes am Zug ist, false sonst
	 */
	public boolean istAAmZug(boolean aLetzterSpieler, int mulde) {
		if (aLetzterSpieler) {
			return mulde == Spielbrett.KALAH_A;
		} else {
			return mulde != Spielbrett.KALAH_B;
		}
	}

	/**
	 * ueberprueft, ob das Spiel beendet ist; uebergeben wird die Information,
	 * ob Spieler A am Zug ist
	 * 
	 * @param istA
	 *            true bedeutet, dass Spieler A am Zug ist; false bedeutet, dass
	 *            Spieler B am Zug ist
	 * @return true, falls das Spiel beendet ist
	 */
	public boolean istSpielBeendet(boolean istA) {

		// das Spiel ist beendet, wenn alle Mulden des Spielers leer
		// sind
		return this.brett.liefereAnzahlKoernerVonSpieler(istA) == 0;
	}

	/**
	 * ueberprueft auf Unentschieden; Voraussetzung: Spiel ist beendet und die
	 * uebrig gebliebenen Koerner wurden in das entsprechende Kalah gepackt
	 * 
	 * @return true, falls das Spiel unentschieden endetde; false sonst
	 */
	public boolean istUnentschieden() {
		return this.brett.liefereAnzahlKoernerInKalahA() == this.brett
				.liefereAnzahlKoernerInKalahB();
	}

	/**
	 * ueberprueft, wer das Spiel gewonnen hat; Voraussetzung: Spiel ist beendet
	 * und nicht unentschieden
	 * 
	 * @return liefert true, wenn Spieler A Sieger ist; liefert false, wenn
	 *         Spieler B Sieger ist;
	 */
	public boolean istSpielerASieger() {
		return this.brett.liefereAnzahlKoernerInKalahA() > this.brett
				.liefereAnzahlKoernerInKalahB();
	}

	/**
	 * liefert das Spielbrett, das von den Spielregeln kontrolliert wird
	 * 
	 * @return das kontrollierte Spielbrett
	 */
	public Spielbrett getBrett() {
		return this.brett;
	}
}
